# Define required providers
terraform {
  required_providers {
    digitalocean = {
      source  = "digitalocean/digitalocean"
      version = "~> 2.0"
    }
  }
}

# Configure the DigitalOcean provider
provider "digitalocean" {
  token = var.do_token
}

# Create the first Droplet
resource "digitalocean_droplet" "droplet_nyc" {
  name     = var.droplet_nyc_name
  region   = "nyc3"
  size     = "s-1vcpu-1gb"
  image    = "debian-12-x64"
  ssh_keys = [data.digitalocean_ssh_key.ssh_key.id]
}

# Create the second Droplet
resource "digitalocean_droplet" "droplet_sfo" {
  name     = var.droplet_sfo_name
  region   = "sfo3"
  size     = "s-1vcpu-1gb"
  image    = "debian-12-x64"
  ssh_keys = [data.digitalocean_ssh_key.ssh_key.id]
}

# Data source for SSH key
data "digitalocean_ssh_key" "ssh_key" {
  name = var.ssh_key_name
}

# TF Firewall
data "digitalocean_firewall" "tf_firewall" {
  firewall_id = var.tf_fw_rule_name
}

# Create a firewall rule for SSH, SMTP, HTTP, and HTTPS
resource "digitalocean_firewall" "tf_firewall" {
  name = var.tf_fw_rule_name 

  # Apply to both Droplets
  droplet_ids = [
    digitalocean_droplet.droplet_nyc.id,
    digitalocean_droplet.droplet_sfo.id
  ]

  # Inbound rules
  inbound_rule {
    protocol         = "icmp"
    source_addresses = ["0.0.0.0/0", "::/0"]
  }

  inbound_rule {
    protocol         = "tcp"
    port_range       = "22" # SSH
    source_addresses = ["0.0.0.0/0", "::/0"] 
  }

  inbound_rule {
    protocol         = "tcp"
    port_range       = "25" # SMTP
    source_addresses = ["0.0.0.0/0", "::/0"]
  }

  inbound_rule {
    protocol         = "udp"
    port_range       = "25" # SMTP
    source_addresses = ["0.0.0.0/0", "::/0"]
  }


  inbound_rule {
    protocol         = "tcp"
    port_range       = "80" # HTTP
    source_addresses = ["0.0.0.0/0", "::/0"]
  }

  inbound_rule {
    protocol         = "tcp"
    port_range       = "443" # HTTPS
    source_addresses = ["0.0.0.0/0", "::/0"]
  }

  # Allow all outbound traffic (default for most setups)
  outbound_rule {
    protocol              = "tcp"
    port_range            = "1-65535"
    destination_addresses = ["0.0.0.0/0", "::/0"]
  }

  outbound_rule {
    protocol              = "udp"
    port_range            = "1-65535"
    destination_addresses = ["0.0.0.0/0", "::/0"]
  }
}

# Set PTR (reverse DNS) records for each Droplet
resource "digitalocean_record" "ptr_droplet_nyc" {
  domain = var.domain
  type   = "A"
  name   = var.droplet_nyc_name 
  value  = digitalocean_droplet.droplet_nyc.ipv4_address
  ttl    = 300
  depends_on = [digitalocean_droplet.droplet_nyc]
}

resource "digitalocean_record" "ptr_droplet_sfo" {
  domain = var.domain
  type   = "A"
  name   = var.droplet_sfo_name 
  value  = digitalocean_droplet.droplet_sfo.ipv4_address
  ttl    = 300
  depends_on = [digitalocean_droplet.droplet_sfo]
}

# Outputs
output "droplet_nyc_ip" {
  value       = digitalocean_droplet.droplet_nyc.ipv4_address
  description = "The public IP address of Droplet NYC"
}

output "droplet_sfo_ip" {
  value       = digitalocean_droplet.droplet_sfo.ipv4_address
  description = "The public IP address of Droplet SFO"
}

output "droplet_nyc_fqdn" {
  value       = "${digitalocean_record.ptr_droplet_nyc.name}.${var.domain}"
  description = "Fully qualified domain name for Droplet NYC"
}

output "droplet_sfo_fqdn" {
  value       = "${digitalocean_record.ptr_droplet_sfo.name}.${var.domain}"
  description = "Fully qualified domain name for Droplet SFO"
}

# Outputs for debugging
output "firewall_id" {
  value       = data.digitalocean_firewall.tf_firewall.id
  description = "ID of the existing firewall"
}

output "firewall_name" {
  value       = data.digitalocean_firewall.tf_firewall.name
  description = "Name of the existing firewall"
}

output "firewall_droplet_ids" {
  value       = data.digitalocean_firewall.tf_firewall.droplet_ids
  description = "Current Droplet IDs associated with the firewall"
}
